@@ -1,6 +1,9 @@
+# TODO: DRY up raise_on_save_failure with attr_accessor_with_default
+# once AS branch is merged in
+
 module DataMapper
   module Resource
-    include Extlib::Assertions
+    include DataMapper::Assertions
     extend Chainable
     extend Deprecate
 
@@ -24,22 +27,55 @@ module DataMapper
       Model.descendants
     end
 
+    # Return if Resource#save should raise an exception on save failures (per-resource)
+    #
+    # This delegates to model.raise_on_save_failure by default.
+    #
+    #   user.raise_on_save_failure  # => false
+    #
+    # @return [Boolean]
+    #   true if a failure in Resource#save should raise an exception
+    #
+    # @api public
+    def raise_on_save_failure
+      if defined?(@raise_on_save_failure)
+        @raise_on_save_failure
+      else
+        model.raise_on_save_failure
+      end
+    end
+
+    # Specify if Resource#save should raise an exception on save failures (per-resource)
+    #
+    # @param [Boolean]
+    #   a boolean that if true will cause Resource#save to raise an exception
+    #
+    # @return [Boolean]
+    #   true if a failure in Resource#save should raise an exception
+    #
+    # @api public
+    def raise_on_save_failure=(raise_on_save_failure)
+      @raise_on_save_failure = raise_on_save_failure
+    end
+
     # Deprecated API for updating attributes and saving Resource
     #
     # @see #update
     #
     # @deprecated
     def update_attributes(attributes = {}, *allowed)
-      assert_update_clean_only(:update_attributes)
+      model      = self.model
+      call_stack = caller[0]
 
-      warn "#{model}#update_attributes is deprecated, use #{model}#update instead (#{caller[0]})"
+      warn "#{model}#update_attributes is deprecated, use #{model}#update instead (#{call_stack})"
 
       if allowed.any?
         warn "specifying allowed in #{model}#update_attributes is deprecated, " \
-          "use Hash#only to filter the attributes in the caller (#{caller[0]})"
+          "use Hash#only to filter the attributes in the caller (#{call_stack})"
         attributes = attributes.only(*allowed)
       end
 
+      assert_update_clean_only(:update_attributes)
       update(attributes)
     end
 
@@ -50,16 +86,41 @@ module DataMapper
       model.extend Model
     end
 
-    # Collection this resource associated with.
-    # Used by SEL.
-    #
-    # @api private
-    attr_writer :collection
-
-    # TODO: document
     # @api public
     alias_method :model, :class
 
+    # Get the persisted state for the resource
+    #
+    # @return [Resource::State]
+    #   the current persisted state for the resource
+    #
+    # @api private
+    def persisted_state
+      @_state ||= Resource::State::Transient.new(self)
+    end
+
+    # Set the persisted state for the resource
+    #
+    # @param [Resource::State]
+    #   the new persisted state for the resource
+    #
+    # @return [undefined]
+    #
+    # @api private
+    def persisted_state=(state)
+      @_state = state
+    end
+
+    # Test if the persisted state is set
+    #
+    # @return [Boolean]
+    #   true if the persisted state is set
+    #
+    # @api private
+    def persisted_state?
+      defined?(@_state) ? true : false
+    end
+
     # Repository this resource belongs to in the context of this collection
     # or of the resource's class.
     #
@@ -69,8 +130,8 @@ module DataMapper
     #
     # @api semipublic
     def repository
-      # only set @repository explicitly when persisted
-      defined?(@repository) ? @repository : model.repository
+      # only set @_repository explicitly when persisted
+      defined?(@_repository) ? @_repository : model.repository
     end
 
     # Retrieve the key(s) for this resource.
@@ -84,18 +145,16 @@ module DataMapper
     #
     # @api public
     def key
-      return @key if defined?(@key)
+      return @_key if defined?(@_key)
 
-      key = model.key(repository_name).map do |property|
+      model_key = model.key(repository_name)
+
+      key = model_key.map do |property|
         original_attributes[property] || (property.loaded?(self) ? property.get!(self) : nil)
       end
 
-      return unless key.all?
-
-      # memoize the key if the Resource is not frozen
-      @key = key unless frozen?
-
-      key
+      # only memoize a valid key
+      @_key = key if model_key.valid?(key)
     end
 
     # Checks if this Resource instance is new
@@ -105,7 +164,7 @@ module DataMapper
     #
     # @api public
     def new?
-      !saved?
+      persisted_state.kind_of?(State::Transient)
     end
 
     # Checks if this Resource instance is saved
@@ -115,7 +174,7 @@ module DataMapper
     #
     # @api public
     def saved?
-      @saved == true
+      persisted_state.kind_of?(State::Persisted)
     end
 
     # Checks if this Resource instance is destroyed
@@ -125,7 +184,7 @@ module DataMapper
     #
     # @api public
     def destroyed?
-      @destroyed == true
+      readonly? && !key.nil?
     end
 
     # Checks if the resource has no changes to save
@@ -135,7 +194,7 @@ module DataMapper
     #
     # @api public
     def clean?
-      !dirty?
+      persisted_state.kind_of?(State::Clean) || persisted_state.kind_of?(State::Immutable)
     end
 
     # Checks if the resource has unsaved changes
@@ -145,15 +204,21 @@ module DataMapper
     #
     # @api public
     def dirty?
-      if original_attributes.any?
-        true
-      elsif new?
-        model.serial || properties.any? { |property| property.default? }
-      else
-        false
+      run_once(true) do
+        dirty_self? || dirty_parents? || dirty_children?
       end
     end
 
+    # Checks if this Resource instance is readonly
+    #
+    # @return [Boolean]
+    #   true if the resource cannot be persisted
+    #
+    # @api public
+    def readonly?
+      persisted_state.kind_of?(State::Immutable)
+    end
+
     # Returns the value of the attribute.
     #
     # Do not read from instance variables directly, but use this method.
@@ -186,10 +251,10 @@ module DataMapper
     #
     # @api public
     def attribute_get(name)
-      properties[name].get(self)
+      persisted_state.get(properties[name])
     end
 
-    alias [] attribute_get
+    alias_method :[], :attribute_get
 
     # Sets the value of the attribute and marks the attribute as dirty
     # if it has been changed so that it may be saved. Do not set from
@@ -223,16 +288,14 @@ module DataMapper
     # @param [Object] value
     #   value to store
     #
-    # @return [Object]
-    #   the value stored at that given attribute, nil if none,
-    #   and default if necessary
+    # @return [undefined]
     #
     # @api public
     def attribute_set(name, value)
-      properties[name].set(self, value)
+      self.persisted_state = persisted_state.set(properties[name], value)
     end
 
-    alias []= attribute_set
+    alias_method :[]=, :attribute_set
 
     # Gets all the attributes of the Resource instance
     #
@@ -247,7 +310,9 @@ module DataMapper
     # @api public
     def attributes(key_on = :name)
       attributes = {}
-      properties.each do |property|
+
+      lazy_load(properties)
+      fields.each do |property|
         if model.public_method_defined?(name = property.name)
           key = case key_on
             when :name  then name
@@ -255,9 +320,10 @@ module DataMapper
             else             property
           end
 
-          attributes[key] = send(name)
+          attributes[key] = __send__(name)
         end
       end
+
       attributes
     end
 
@@ -271,32 +337,42 @@ module DataMapper
     #
     # @api public
     def attributes=(attributes)
+      model = self.model
       attributes.each do |name, value|
         case name
           when String, Symbol
             if model.public_method_defined?(setter = "#{name}=")
-              send(setter, value)
+              __send__(setter, value)
             else
               raise ArgumentError, "The attribute '#{name}' is not accessible in #{model}"
             end
           when Associations::Relationship, Property
-            name.set(self, value)
+            self.persisted_state = persisted_state.set(name, value)
         end
       end
     end
 
     # Reloads association and all child association
     #
+    # This is accomplished by resetting the Resource key to it's
+    # original value, and then removing all the ivars for properties
+    # and relationships.  On the next access of those ivars, the
+    # resource will eager load what it needs.  While this is more of
+    # a lazy reload, it should result is more consistent behavior
+    # since no cached results will remain from the initial load.
+    #
     # @return [Resource]
     #   the receiver, the current Resource instance
     #
     # @api public
     def reload
-      if saved?
-        eager_load(loaded_properties)
-        child_relationships.each { |relationship| relationship.get!(self).reload }
+      if key
+        reset_key
+        clear_subjects
       end
 
+      self.persisted_state = persisted_state.rollback
+
       self
     end
 
@@ -309,12 +385,10 @@ module DataMapper
     #   true if resource and storage state match
     #
     # @api public
-    chainable do
-      def update(attributes = {})
-        assert_update_clean_only(:update)
-        self.attributes = attributes
-        save
-      end
+    def update(attributes)
+      assert_update_clean_only(:update)
+      self.attributes = attributes
+      save
     end
 
     # Updates attributes and saves this Resource instance, bypassing hooks
@@ -326,7 +400,7 @@ module DataMapper
     #   true if resource and storage state match
     #
     # @api public
-    def update!(attributes = {})
+    def update!(attributes)
       assert_update_clean_only(:update!)
       self.attributes = attributes
       save!
@@ -338,10 +412,11 @@ module DataMapper
     #   true if Resource instance and all associations were saved
     #
     # @api public
-    chainable do
-      def save
-        save_parents && save_self && save_children
-      end
+    def save
+      assert_not_destroyed(:save)
+      retval = _save
+      assert_save_successful(:save, retval)
+      retval
     end
 
     # Save the instance and loaded, dirty associations to the data-store, bypassing hooks
@@ -351,7 +426,10 @@ module DataMapper
     #
     # @api public
     def save!
-      save_parents(false) && save_self(false) && save_children(false)
+      assert_not_destroyed(:save!)
+      retval = _save(false)
+      assert_save_successful(:save!, retval)
+      retval
     end
 
     # Destroy the instance, remove it from the repository
@@ -360,10 +438,14 @@ module DataMapper
     #   true if resource was destroyed
     #
     # @api public
-    chainable do
-      def destroy
-        destroy!
+    def destroy
+      return true if destroyed?
+      catch :halt do
+        before_destroy_hook
+        _destroy
+        after_destroy_hook
       end
+      destroyed?
     end
 
     # Destroy the instance, remove it from the repository, bypassing hooks
@@ -373,21 +455,16 @@ module DataMapper
     #
     # @api public
     def destroy!
-      if saved? && repository.delete(Collection.new(query, [ self ])) == 1
-        @destroyed = true
-        @collection.delete(self) if @collection
-        reset
-        freeze
-      end
-
+      return true if destroyed?
+      _destroy(false)
       destroyed?
     end
 
     # Compares another Resource for equality
     #
-    # Resource is equal to +other+ if they are the same object (identity)
-    # or if they are both of the *same model* and all of their attributes
-    # are equivalent
+    # Resource is equal to +other+ if they are the same object
+    # (identical object_id) or if they are both of the *same model* and
+    # all of their attributes are equivalent
     #
     # @param [Resource] other
     #   the other Resource to compare with
@@ -403,9 +480,8 @@ module DataMapper
 
     # Compares another Resource for equivalency
     #
-    # Resource is equal to +other+ if they are the same object (identity)
-    # or if they are both of the *same base model* and all of their attributes
-    # are equivalent
+    # Resource is equivalent to +other+ if they are the same object
+    # (identical object_id) or all of their attribute are equivalent
     #
     # @param [Resource] other
     #   the other Resource to compare with
@@ -416,8 +492,7 @@ module DataMapper
     # @api public
     def ==(other)
       return true if equal?(other)
-      other.respond_to?(:model)                       &&
-      model.base_model.equal?(other.model.base_model) &&
+      return false unless other.kind_of?(Resource) && model.base_model.equal?(other.model.base_model)
       cmp?(other, :==)
     end
 
@@ -433,15 +508,15 @@ module DataMapper
     #
     # @api public
     def <=>(other)
+      model = self.model
       unless other.kind_of?(model.base_model)
-        raise ArgumentError, "Cannot compare a #{other.model} instance with a #{model} instance"
+        raise ArgumentError, "Cannot compare a #{other.class} instance with a #{model} instance"
       end
-      cmp = 0
       model.default_order(repository_name).each do |direction|
         cmp = direction.get(self) <=> direction.get(other)
-        break if cmp != 0
+        return cmp if cmp.nonzero?
       end
-      cmp
+      0
     end
 
     # Returns hash value of the object.
@@ -488,7 +563,11 @@ module DataMapper
     #
     # @api semipublic
     def original_attributes
-      @original_attributes ||= {}
+      if persisted_state.respond_to?(:original_attributes)
+        persisted_state.original_attributes.dup.freeze
+      else
+        {}.freeze
+      end
     end
 
     # Checks if an attribute has been loaded from the repository
@@ -537,98 +616,133 @@ module DataMapper
       dirty_attributes = {}
 
       original_attributes.each_key do |property|
-        dirty_attributes[property] = property.value(property.get!(self))
+        next unless property.respond_to?(:value)
+        dirty_attributes[property] = property.dump(property.get!(self))
       end
 
       dirty_attributes
     end
 
-    # Saves the resource
+    # Returns the Collection the Resource is associated with
     #
-    # @return [Boolean]
-    #   true if the resource was successfully saved
-    #
-    # @api semipublic
-    def save_self(safe = true)
-      if safe
-        new? ? create_hook : update_hook
-      else
-        new? ? _create : _update
-      end
-    end
-
-    # Saves the parent resources
-    #
-    # @return [Boolean]
-    #   true if the parents were successfully saved
-    #
-    # @api private
-    def save_parents(safe = true)
-      parent_relationships.all? do |relationship|
-        parent = relationship.get!(self)
-        if parent.dirty? ? parent.save_parents(safe) && parent.save_self(safe) : parent.saved?
-          relationship.set(self, parent)  # set the FK values
-        end
-      end
-    end
-
-    # Saves the children resources
-    #
-    # @return [Boolean]
-    #   true if the children were successfully saved
-    #
-    # @api private
-    def save_children(safe = true)
-      child_relationships.all? do |relationship|
-        association = relationship.get!(self)
-        safe ? association.save : association.save!
-      end
-    end
-
-    # Reset the Resource to a similar state as a new record:
-    # removes it from identity map and clears original property
-    # values (thus making all properties non dirty)
-    #
-    # @api private
-    def reset
-      @saved = false
-      identity_map.delete(key)
-      original_attributes.clear
-      self
-    end
-
-    # Gets a Collection with the current Resource instance as its only member
-    #
-    # @return [Collection, FalseClass]
-    #   nil if this is a new record,
-    #   otherwise a Collection with self as its only member
+    # @return [nil]
+    #    nil if this is a new record
+    # @return [Collection]
+    #   a Collection that self belongs to
     #
     # @api private
     def collection
-      return @collection if @collection || new? || frozen?
-      @collection = Collection.new(query, [ self ])
+      return @_collection if (@_collection && @_collection.query.conditions.matches?(self)) || new? || readonly?
+      collection_for_self
+    end
+
+    # Associates a Resource to a Collection
+    #
+    # @param [Collection, nil] collection
+    #   the collection to associate the resource with
+    #
+    # @return [nil]
+    #    nil if this is a new record
+    # @return [Collection]
+    #   a Collection that self belongs to
+    #
+    # @api private
+    def collection=(collection)
+      @_collection = collection
+    end
+
+    # Return a collection including the current resource only
+    #
+    # @return [Collection]
+    #   a collection containing self
+    #
+    # @api private
+    def collection_for_self
+      Collection.new(query, [ self ])
+    end
+
+    # Returns a Query that will match the resource
+    #
+    # @return [Query]
+    #   Query that will match the resource
+    #
+    # @api semipublic
+    def query
+      repository.new_query(model, :fields => fields, :conditions => conditions)
     end
 
     protected
 
-    # Method for hooking callbacks on resource creation
+    # Method for hooking callbacks before resource saving
     #
-    # @return [Boolean]
-    #   true if the create was successful, false if not
+    # @return [undefined]
     #
     # @api private
-    def create_hook
-      _create
+    def before_save_hook
+      execute_hooks_for(:before, :save)
     end
 
-    # Method for hooking callbacks on resource updates
+    # Method for hooking callbacks after resource saving
     #
-    # @return [Boolean]
-    #   true if the update was successful, false if not
+    # @return [undefined]
     #
     # @api private
-    def update_hook
-      _update
+    def after_save_hook
+      execute_hooks_for(:after, :save)
+    end
+
+    # Method for hooking callbacks before resource creation
+    #
+    # @return [undefined]
+    #
+    # @api private
+    def before_create_hook
+      execute_hooks_for(:before, :create)
+    end
+
+    # Method for hooking callbacks after resource creation
+    #
+    # @return [undefined]
+    #
+    # @api private
+    def after_create_hook
+      execute_hooks_for(:after, :create)
+    end
+
+    # Method for hooking callbacks before resource updating
+    #
+    # @return [undefined]
+    #
+    # @api private
+    def before_update_hook
+      execute_hooks_for(:before, :update)
+    end
+
+    # Method for hooking callbacks after resource updating
+    #
+    # @return [undefined]
+    #
+    # @api private
+    def after_update_hook
+      execute_hooks_for(:after, :update)
+    end
+
+    # Method for hooking callbacks before resource destruction
+    #
+    # @return [undefined]
+    #
+    # @api private
+    def before_destroy_hook
+      execute_hooks_for(:before, :destroy)
+    end
+
+    # Method for hooking callbacks after resource destruction
+    #
+    # @return [undefined]
+    #
+    # @api private
+    def after_destroy_hook
+      execute_hooks_for(:after, :destroy)
     end
 
     private
@@ -642,8 +756,17 @@ module DataMapper
     #   attribute values used in the new instance
     #
     # @api public
-    def initialize(attributes = {}, &block) # :nodoc:
-      self.attributes = attributes
+    def initialize(attributes = nil) # :nodoc:
+      self.attributes = attributes if attributes
+    end
+
+    # @api private
+    def initialize_copy(original)
+      instance_variables.each do |ivar|
+        instance_variable_set(ivar, instance_variable_get(ivar).try_dup)
+      end
+
+      self.persisted_state = persisted_state.class.new(self)
     end
 
     # Returns name of the repository this object
@@ -682,11 +805,21 @@ module DataMapper
     # @return [IdentityMap]
     #   identity map of repository this object was loaded from
     #
-    # @api semipublic
+    # @api private
     def identity_map
       repository.identity_map(model)
     end
 
+    # @api private
+    def add_to_identity_map
+      identity_map[key] = self
+    end
+
+    # @api private
+    def remove_from_identity_map
+      identity_map.delete(key)
+    end
+
     # Fetches all the names of the attributes that have been loaded,
     # even if they are lazy but have been called
     #
@@ -694,57 +827,97 @@ module DataMapper
     #   names of attributes that have been loaded
     #
     # @api private
-    def loaded_properties
-      properties.select { |property| property.loaded?(self) }
+    def fields
+      properties.select do |property|
+        property.loaded?(self) || (new? && property.default?)
+      end
+    end
+
+    # Reset the key to the original value
+    #
+    # @return [undefined]
+    #
+    # @api private
+    def reset_key
+      properties.key.zip(key) do |property, value|
+        property.set!(self, value)
+      end
+    end
+
+    # Remove all the ivars for properties and relationships
+    #
+    # @return [undefined]
+    #
+    # @api private
+    def clear_subjects
+      model_properties = properties
+
+      (model_properties - model_properties.key | relationships.values).each do |subject|
+        next unless subject.loaded?(self)
+        remove_instance_variable(subject.instance_variable_name)
+      end
     end
 
     # Lazy loads attributes not yet loaded
     #
-    # @param [Array<Property>] fields
+    # @param [Array<Property>] properties
     #   the properties to reload
     #
     # @return [self]
     #
     # @api private
-    def lazy_load(fields)
-      eager_load(fields - loaded_properties)
+    def lazy_load(properties)
+      eager_load(properties - fields)
     end
 
     # Reloads specified attributes
     #
-    # @param [Array<Property>] fields
+    # @param [Array<Property>] properties
     #   the properties to reload
     #
     # @return [Resource]
     #   the receiver, the current Resource instance
     #
     # @api private
-    def eager_load(fields)
-      unless fields.empty? || new?
-        collection.reload(:fields => fields)
+    def eager_load(properties)
+      unless properties.empty? || key.nil? || collection.nil?
+        # set an initial value to prevent recursive lazy loads
+        properties.each { |property| property.set!(self, nil) }
+
+        collection.reload(:fields => properties)
       end
 
       self
     end
 
-    # Gets a Query that will return this Resource instance
+    # Return conditions to match the Resource
     #
-    # @return [Query]
-    #   Query that will retrieve this Resource instance
+    # @return [Hash]
+    #   query conditions
     #
     # @api private
-    def query
-      Query.new(repository, model, model.key_conditions(repository, key))
+    def conditions
+      key = self.key
+      if key
+        model.key_conditions(repository, key)
+      else
+        conditions = {}
+        properties.each do |property|
+          next unless property.loaded?(self)
+          conditions[property] = property.get!(self)
+        end
+        conditions
+      end
     end
 
-    # TODO: document
     # @api private
     def parent_relationships
       parent_relationships = []
 
       relationships.each_value do |relationship|
-        next unless relationship.respond_to?(:resource_for) && relationship.loaded?(self)
-        next unless relationship.get(self)
+        next unless relationship.respond_to?(:resource_for)
+        set_default_value(relationship)
+        next unless relationship.loaded?(self) && relationship.get!(self)
 
         parent_relationships << relationship
       end
@@ -762,10 +935,9 @@ module DataMapper
       child_relationships = []
 
       relationships.each_value do |relationship|
-        next unless relationship.respond_to?(:collection_for) && relationship.loaded?(self)
-
-        association = relationship.get!(self)
-        next unless association.loaded? || association.head.any? || association.tail.any?
+        next unless relationship.respond_to?(:collection_for)
+        set_default_value(relationship)
+        next unless relationship.loaded?(self)
 
         child_relationships << relationship
       end
@@ -777,80 +949,167 @@ module DataMapper
       many_to_many + other
     end
 
-    # Creates the resource with default values
-    #
-    # If resource is not dirty or a new (not yet saved),
-    # this method returns false
-    #
-    # On successful save identity map of the repository is
-    # updated
-    #
-    # Needs to be a protected method so that it is hookable
-    #
-    # The primary purpose of this method is to allow before :create
-    # hooks to fire at a point just before/after resource creation
-    #
-    # @return [Boolean]
-    #   true if the receiver was successfully created
-    #
     # @api private
-    def _create
-      # Can't create a resource that is not dirty and doesn't have serial keys
-      return false if new? && !dirty?
-
-      # set defaults for new resource
-      properties.each do |property|
-        unless property.serial? || property.loaded?(self)
-          property.set(self, property.default_for(self))
-        end
-      end
-
-      repository.create([ self ])
-
-      @repository = repository
-      @saved      = true
-
-      original_attributes.clear
-
-      identity_map[key] = self
-
-      true
+    def parent_associations
+      parent_relationships.map { |relationship| relationship.get!(self) }
     end
 
-    # Updates resource state
+    # @api private
+    def child_associations
+      child_relationships.map { |relationship| relationship.get_collection(self) }
+    end
+
+    # Commit the persisted state
     #
-    # The primary purpose of this method is to allow before :update
-    # hooks to fire at a point just before/after resource update whether
-    # it is the result of Resource#save, or using Resource#update
-    #
-    # @return [Boolean]
-    #   true if the receiver was successfully created
+    # @return [undefined]
     #
     # @api private
-    def _update
-      dirty_attributes = self.dirty_attributes
+    def _persist
+      self.persisted_state = persisted_state.commit
+    end
 
-      if dirty_attributes.empty?
-        true
-      elsif dirty_attributes.any? { |property, value| !property.nullable? && value.nil? }
-        false
-      else
-        # remove from the identity map
-        identity_map.delete(key)
-
-        return false unless repository.update(dirty_attributes, Collection.new(query, [ self ])) == 1
-
-        # remove the cached key in case it is updated
-        remove_instance_variable(:@key)
-
-        original_attributes.clear
-
-        identity_map[key] = self
-
-        true
+    # This method executes the hooks before and after resource creation
+    #
+    # @return [Boolean]
+    #
+    # @see Resource#_create
+    #
+    # @api private
+    def create_with_hooks
+      catch :halt do
+        before_save_hook
+        before_create_hook
+        _persist
+        after_create_hook
+        after_save_hook
       end
     end
 
+    # This method executes the hooks before and after resource updating
+    #
+    # @return [Boolean]
+    #
+    # @see Resource#_update
+    #
+    # @api private
+    def update_with_hooks
+      catch :halt do
+        before_save_hook
+        before_update_hook
+        _persist
+        after_update_hook
+        after_save_hook
+      end
+    end
+
+    # Destroy the resource
+    #
+    # @return [undefined]
+    #
+    # @api private
+    def _destroy(execute_hooks = true)
+      self.persisted_state = persisted_state.delete
+      _persist
+    end
+
+    # @api private
+    def _save(execute_hooks = true)
+      run_once(true) do
+        save_parents(execute_hooks) && save_self(execute_hooks) && save_children(execute_hooks)
+      end
+    end
+
+    # Saves the resource
+    #
+    # @return [Boolean]
+    #   true if the resource was successfully saved
+    #
+    # @api semipublic
+    def save_self(execute_hooks = true)
+      # short-circuit if the resource is not dirty
+      return saved? unless dirty_self?
+
+      if execute_hooks
+        new? ? create_with_hooks : update_with_hooks
+      else
+        _persist
+      end
+      clean?
+    end
+
+    # Saves the parent resources
+    #
+    # @return [Boolean]
+    #   true if the parents were successfully saved
+    #
+    # @api private
+    def save_parents(execute_hooks)
+      run_once(true) do
+        parent_relationships.map do |relationship|
+          parent = relationship.get!(self)
+
+          if parent.__send__(:save_parents, execute_hooks) && parent.__send__(:save_self, execute_hooks)
+            relationship.set(self, parent)  # set the FK values
+          end
+        end.all?
+      end
+    end
+
+    # Saves the children resources
+    #
+    # @return [Boolean]
+    #   true if the children were successfully saved
+    #
+    # @api private
+    def save_children(execute_hooks)
+      child_associations.map do |association|
+        association.__send__(execute_hooks ? :save : :save!)
+      end.all?
+    end
+
+    # Checks if the resource has unsaved changes
+    #
+    # @return [Boolean]
+    #  true if the resource has unsaged changes
+    #
+    # @api semipublic
+    def dirty_self?
+      if original_attributes.any?
+        true
+      elsif new?
+        !model.serial.nil? || properties.any? { |property| property.default? }
+      else
+        false
+      end
+    end
+
+    # Checks if the parents have unsaved changes
+    #
+    # @return [Boolean]
+    #  true if the parents have unsaved changes
+    #
+    # @api private
+    def dirty_parents?
+      run_once(false) do
+        parent_associations.any? do |association|
+          association.__send__(:dirty_self?) || association.__send__(:dirty_parents?)
+        end
+      end
+    end
+
+    # Checks if the children have unsaved changes
+    #
+    # @param [Hash] resources
+    #   resources that have already been tested
+    #
+    # @return [Boolean]
+    #  true if the children have unsaved changes
+    #
+    # @api private
+    def dirty_children?
+      child_associations.any? { |association| association.dirty? }
+    end
+
     # Return true if +other+'s is equivalent or equal to +self+'s
     #
     # @param [Resource] other
@@ -863,17 +1122,38 @@ module DataMapper
     #
     # @api private
     def cmp?(other, operator)
-      return false unless key.send(operator, other.key)
-      return true if repository.send(operator, other.repository) && !dirty? && !other.dirty?
+      return false unless repository.send(operator, other.repository) &&
+                          key.send(operator, other.key)
 
-      # get all the loaded and non-loaded properties that are not keys,
-      # since the key comparison was performed earlier
-      loaded, not_loaded = properties.select { |property| !property.key? }.partition do |property|
-        property.loaded?(self) && property.loaded?(other)
+      if saved? && other.saved?
+        # if dirty attributes match then they are the same resource
+        dirty_attributes == other.dirty_attributes
+      else
+        # compare properties for unsaved resources
+        properties.all? do |property|
+          __send__(property.name).send(operator, other.__send__(property.name))
+        end
       end
+    end
 
-      # check all loaded properties, and then all unloaded properties
-      (loaded + not_loaded).all? { |property| property.get(self).send(operator, property.get(other)) }
+    # @api private
+    def set_default_value(subject)
+      return unless persisted_state.respond_to?(:set_default_value, true)
+      persisted_state.__send__(:set_default_value, subject)
+    end
+
+    # Execute all the queued up hooks for a given type and name
+    #
+    # @param [Symbol] type
+    #   the type of hook to execute (before or after)
+    # @param [Symbol] name
+    #   the name of the hook to execute
+    #
+    # @return [undefined]
+    #
+    # @api private
+    def execute_hooks_for(type, name)
+      model.hooks[name][type].each { |hook| hook.call(self) }
     end
 
     # Raises an exception if #update is performed on a dirty resource
@@ -888,8 +1168,72 @@ module DataMapper
     #
     # @api private
     def assert_update_clean_only(method)
-      if original_attributes.any?
-        raise UpdateConflictError, "#{model}##{method} cannot be called on a dirty resource"
+      if dirty?
+        raise UpdateConflictError, "#{model}##{method} cannot be called on a #{new? ? 'new' : 'dirty'} resource"
+      end
+    end
+
+    # Raises an exception if #save is performed on a destroyed resource
+    #
+    # @param [Symbol] method
+    #   the name of the method to use in the exception
+    #
+    # @return [undefined]
+    #
+    # @raise [PersistenceError]
+    #   raise if the resource is destroyed
+    #
+    # @api private
+    def assert_not_destroyed(method)
+      if destroyed?
+        raise PersistenceError, "#{model}##{method} cannot be called on a destroyed resource"
+      end
+    end
+
+    # Raises an exception if #save returns false
+    #
+    # @param [Symbol] method
+    #   the name of the method to use in the exception
+    # @param [Boolean] save_result
+    #   the result of the #save call
+    #
+    # @return [undefined]
+    #
+    # @raise [SaveFailureError]
+    #   raise if the resource was not saved
+    #
+    # @api private
+    def assert_save_successful(method, save_retval)
+      if save_retval != true && raise_on_save_failure
+        raise SaveFailureError.new("#{model}##{method} returned #{save_retval.inspect}, #{model} was not saved", self)
+      end
+    end
+
+    # Prevent a method from being in the stack more than once
+    #
+    # The purpose of this method is to prevent SystemStackError from
+    # being thrown from methods from encountering infinite recursion
+    # when called on resources having circular dependencies.
+    #
+    # @param [Object] default
+    #   default return value
+    #
+    # @yield The block of code to run once
+    #
+    # @return [Object]
+    #   block return value
+    #
+    # @api private
+    def run_once(default)
+      caller_method = Kernel.caller(1).first[/`([^'?!]+)[?!]?'/, 1]
+      sentinel      = "@_#{caller_method}_sentinel"
+      return instance_variable_get(sentinel) if instance_variable_defined?(sentinel)
+
+      begin
+        instance_variable_set(sentinel, default)
+        yield
+      ensure
+        remove_instance_variable(sentinel)
       end
     end
   end # module Resource
