@@ -1,17 +1,19 @@
+require 'dm-core/type'
+
 module DataMapper
   module Types
     class Discriminator < Type
       primitive Class
       default   lambda { |resource, property| resource.model }
-      nullable  false
+      required  true
 
-      # TODO: document
       # @api private
       def self.bind(property)
         repository_name = property.repository_name
         model           = property.model
+        property_name   = property.name
 
-        model.default_scope(repository_name).update(property.name => model.descendants)
+        model.default_scope(repository_name).update(property_name => model.descendants.dup << model)
 
         model.class_eval <<-RUBY, __FILE__, __LINE__ + 1
           extend Chainable
@@ -38,7 +40,7 @@ module DataMapper
             private
 
             def set_discriminator_scope_for(model)
-              model.default_scope(#{repository_name.inspect}).update(#{property.name.inspect} => model.descendants)
+              model.default_scope(#{repository_name.inspect}).update(#{property_name.inspect} => model.descendants.dup << model)
             end
           end
         RUBY
